@@ -5,93 +5,47 @@
 
 #include "nsAccUtils.h"
 
-#include "Accessible-inl.h"
+#include "LocalAccessible-inl.h"
+#include "AccAttributes.h"
 #include "ARIAMap.h"
 #include "nsAccessibilityService.h"
 #include "nsCoreUtils.h"
 #include "DocAccessible.h"
+#include "DocAccessibleParent.h"
 #include "HyperTextAccessible.h"
 #include "nsIAccessibleTypes.h"
 #include "Role.h"
 #include "States.h"
 #include "TextLeafAccessible.h"
 
+#include "nsIBaseWindow.h"
+#include "nsIDocShellTreeOwner.h"
 #include "nsIDOMXULContainerElement.h"
-#include "nsIPersistentProperties2.h"
+#include "nsISimpleEnumerator.h"
 #include "mozilla/a11y/PDocAccessibleChild.h"
+#include "mozilla/a11y/RemoteAccessible.h"
+#include "mozilla/dom/Document.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/StaticPrefs_accessibility.h"
 #include "nsAccessibilityService.h"
 
 using namespace mozilla;
 using namespace mozilla::a11y;
 
-void nsAccUtils::GetAccAttr(nsIPersistentProperties* aAttributes,
-                            nsAtom* aAttrName, nsAString& aAttrValue) {
-  aAttrValue.Truncate();
-
-  aAttributes->GetStringProperty(nsAtomCString(aAttrName), aAttrValue);
-}
-
-void nsAccUtils::SetAccAttr(nsIPersistentProperties* aAttributes,
-                            nsAtom* aAttrName, const nsAString& aAttrValue) {
-  nsAutoString oldValue;
-  aAttributes->SetStringProperty(nsAtomCString(aAttrName), aAttrValue,
-                                 oldValue);
-}
-
-void nsAccUtils::SetAccAttr(nsIPersistentProperties* aAttributes,
-                            nsAtom* aAttrName, nsAtom* aAttrValue) {
-  nsAutoString oldValue;
-  aAttributes->SetStringProperty(nsAtomCString(aAttrName),
-                                 nsAtomString(aAttrValue), oldValue);
-}
-
-void nsAccUtils::SetAccGroupAttrs(nsIPersistentProperties* aAttributes,
-                                  int32_t aLevel, int32_t aSetSize,
-                                  int32_t aPosInSet) {
+void nsAccUtils::SetAccGroupAttrs(AccAttributes* aAttributes, int32_t aLevel,
+                                  int32_t aSetSize, int32_t aPosInSet) {
   nsAutoString value;
 
   if (aLevel) {
-    value.AppendInt(aLevel);
-    SetAccAttr(aAttributes, nsGkAtoms::level, value);
+    aAttributes->SetAttribute(nsGkAtoms::level, aLevel);
   }
 
   if (aSetSize && aPosInSet) {
-    value.Truncate();
-    value.AppendInt(aPosInSet);
-    SetAccAttr(aAttributes, nsGkAtoms::posinset, value);
-
-    value.Truncate();
-    value.AppendInt(aSetSize);
-    SetAccAttr(aAttributes, nsGkAtoms::setsize, value);
+    aAttributes->SetAttribute(nsGkAtoms::posinset, aPosInSet);
+    aAttributes->SetAttribute(nsGkAtoms::setsize, aSetSize);
   }
 }
 
-int32_t nsAccUtils::GetDefaultLevel(const Accessible* aAccessible) {
-  roles::Role role = aAccessible->Role();
-
-  if (role == roles::OUTLINEITEM) return 1;
-
-  if (role == roles::ROW) {
-    Accessible* parent = aAccessible->Parent();
-    // It is a row inside flatten treegrid. Group level is always 1 until it
-    // is overriden by aria-level attribute.
-    if (parent && parent->Role() == roles::TREE_TABLE) return 1;
-  }
-
-  return 0;
-}
-
-int32_t nsAccUtils::GetARIAOrDefaultLevel(const Accessible* aAccessible) {
-  int32_t level = 0;
-  nsCoreUtils::GetUIntAttr(aAccessible->GetContent(), nsGkAtoms::aria_level,
-                           &level);
-
-  if (level != 0) return level;
-
-  return GetDefaultLevel(aAccessible);
-}
-
 int32_t nsAccUtils::GetLevelForXULContainerItem(nsIContent* aContent) {
   nsCOMPtr<nsIDOMXULContainerItemElement> item =
       aContent->AsElement()->AsXULContainerItem();
@@ -115,8 +69,8 @@ int32_t nsAccUtils::GetLevelForXULContainerItem(nsIContent* aContent) {
   return level;
 }
 
-void nsAccUtils::SetLiveContainerAttributes(
-    nsIPersistentProperties* aAttributes, nsIContent* aStartContent) {
+void nsAccUtils::SetLiveContainerAttributes(AccAttributes* aAttributes,
+                                            nsIContent* aStartContent) {
   nsAutoString live, relevant, busy;
   dom::Document* doc = aStartContent->GetComposedDoc();
   if (!doc) {
@@ -129,8 +83,10 @@ void nsAccUtils::SetLiveContainerAttributes(
     if (relevant.IsEmpty() &&
         HasDefinedARIAToken(ancestor, nsGkAtoms::aria_relevant) &&
         ancestor->AsElement()->GetAttr(kNameSpaceID_None,
-                                       nsGkAtoms::aria_relevant, relevant))
-      SetAccAttr(aAttributes, nsGkAtoms::containerRelevant, relevant);
+                                       nsGkAtoms::aria_relevant, relevant)) {
+      aAttributes->SetAttributeStringCopy(nsGkAtoms::containerRelevant,
+                                          relevant);
+    }
 
     // container-live, and container-live-role attributes
     if (live.IsEmpty()) {
@@ -144,15 +100,15 @@ void nsAccUtils::SetLiveContainerAttributes(
       } else if (role) {
         GetLiveAttrValue(role->liveAttRule, live);
       } else if (nsStaticAtom* value = GetAccService()->MarkupAttribute(
-                     ancestor, nsGkAtoms::live)) {
+                     ancestor, nsGkAtoms::aria_live)) {
         value->ToString(live);
       }
 
       if (!live.IsEmpty()) {
-        SetAccAttr(aAttributes, nsGkAtoms::containerLive, live);
+        aAttributes->SetAttributeStringCopy(nsGkAtoms::containerLive, live);
         if (role) {
-          SetAccAttr(aAttributes, nsGkAtoms::containerLiveRole,
-                     role->ARIARoleString());
+          aAttributes->SetAttribute(nsGkAtoms::containerLiveRole,
+                                    role->roleAtom);
         }
       }
     }
@@ -161,15 +117,15 @@ void nsAccUtils::SetLiveContainerAttributes(
     if (ancestor->IsElement() && ancestor->AsElement()->AttrValueIs(
                                      kNameSpaceID_None, nsGkAtoms::aria_atomic,
                                      nsGkAtoms::_true, eCaseMatters)) {
-      SetAccAttr(aAttributes, nsGkAtoms::containerAtomic,
-                 NS_LITERAL_STRING("true"));
+      aAttributes->SetAttribute(nsGkAtoms::containerAtomic, true);
     }
 
     // container-busy attribute
     if (busy.IsEmpty() && HasDefinedARIAToken(ancestor, nsGkAtoms::aria_busy) &&
         ancestor->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::aria_busy,
-                                       busy))
-      SetAccAttr(aAttributes, nsGkAtoms::containerBusy, busy);
+                                       busy)) {
+      aAttributes->SetAttributeStringCopy(nsGkAtoms::containerBusy, busy);
+    }
 
     if (ancestor == topEl) {
       break;
@@ -231,20 +187,21 @@ nsStaticAtom* nsAccUtils::NormalizeARIAToken(dom::Element* aElement,
   return nullptr;
 }
 
-Accessible* nsAccUtils::GetSelectableContainer(Accessible* aAccessible,
-                                               uint64_t aState) {
+LocalAccessible* nsAccUtils::GetSelectableContainer(
+    LocalAccessible* aAccessible, uint64_t aState) {
   if (!aAccessible) return nullptr;
 
   if (!(aState & states::SELECTABLE)) return nullptr;
 
-  Accessible* parent = aAccessible;
-  while ((parent = parent->Parent()) && !parent->IsSelect()) {
+  LocalAccessible* parent = aAccessible;
+  while ((parent = parent->LocalParent()) && !parent->IsSelect()) {
     if (parent->Role() == roles::PANE) return nullptr;
   }
   return parent;
 }
 
-bool nsAccUtils::IsDOMAttrTrue(const Accessible* aAccessible, nsAtom* aAttr) {
+bool nsAccUtils::IsDOMAttrTrue(const LocalAccessible* aAccessible,
+                               nsAtom* aAttr) {
   dom::Element* el = aAccessible->Elm();
   return el && el->AttrValueIs(kNameSpaceID_None, aAttr, nsGkAtoms::_true,
                                eCaseMatters);
@@ -273,33 +230,40 @@ Accessible* nsAccUtils::TableFor(Accessible* aRow) {
   return nullptr;
 }
 
+LocalAccessible* nsAccUtils::TableFor(LocalAccessible* aRow) {
+  Accessible* table = TableFor(static_cast<Accessible*>(aRow));
+  return table ? table->AsLocal() : nullptr;
+}
+
 HyperTextAccessible* nsAccUtils::GetTextContainer(nsINode* aNode) {
   // Get text accessible containing the result node.
   DocAccessible* doc = GetAccService()->GetDocAccessible(aNode->OwnerDoc());
-  Accessible* accessible = doc ? doc->GetAccessibleOrContainer(aNode) : nullptr;
+  LocalAccessible* accessible =
+      doc ? doc->GetAccessibleOrContainer(aNode) : nullptr;
   if (!accessible) return nullptr;
 
   do {
     HyperTextAccessible* textAcc = accessible->AsHyperText();
     if (textAcc) return textAcc;
 
-    accessible = accessible->Parent();
+    accessible = accessible->LocalParent();
   } while (accessible);
 
   return nullptr;
 }
 
-nsIntPoint nsAccUtils::ConvertToScreenCoords(int32_t aX, int32_t aY,
-                                             uint32_t aCoordinateType,
-                                             Accessible* aAccessible) {
-  nsIntPoint coords(aX, aY);
+LayoutDeviceIntPoint nsAccUtils::ConvertToScreenCoords(
+    int32_t aX, int32_t aY, uint32_t aCoordinateType, Accessible* aAccessible) {
+  LayoutDeviceIntPoint coords(aX, aY);
 
   switch (aCoordinateType) {
+    // Regardless of coordinate type, the coords returned
+    // are in dev pixels.
     case nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE:
       break;
 
     case nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE: {
-      coords += nsCoreUtils::GetScreenCoordsForWindow(aAccessible->GetNode());
+      coords += GetScreenCoordsForWindow(aAccessible);
       break;
     }
 
@@ -319,19 +283,20 @@ void nsAccUtils::ConvertScreenCoordsTo(int32_t* aX, int32_t* aY,
                                        uint32_t aCoordinateType,
                                        Accessible* aAccessible) {
   switch (aCoordinateType) {
+    // Regardless of coordinate type, the values returned for
+    // aX and aY are in dev pixels.
     case nsIAccessibleCoordinateType::COORDTYPE_SCREEN_RELATIVE:
       break;
 
     case nsIAccessibleCoordinateType::COORDTYPE_WINDOW_RELATIVE: {
-      nsIntPoint coords =
-          nsCoreUtils::GetScreenCoordsForWindow(aAccessible->GetNode());
+      LayoutDeviceIntPoint coords = GetScreenCoordsForWindow(aAccessible);
       *aX -= coords.x;
       *aY -= coords.y;
       break;
     }
 
     case nsIAccessibleCoordinateType::COORDTYPE_PARENT_RELATIVE: {
-      nsIntPoint coords = GetScreenCoordsForParent(aAccessible);
+      LayoutDeviceIntPoint coords = GetScreenCoordsForParent(aAccessible);
       *aX -= coords.x;
       *aY -= coords.y;
       break;
@@ -342,25 +307,54 @@ void nsAccUtils::ConvertScreenCoordsTo(int32_t* aX, int32_t* aY,
   }
 }
 
-nsIntPoint nsAccUtils::GetScreenCoordsForParent(Accessible* aAccessible) {
-  Accessible* parent = aAccessible->Parent();
-  if (!parent) return nsIntPoint(0, 0);
+LayoutDeviceIntPoint nsAccUtils::GetScreenCoordsForParent(
+    Accessible* aAccessible) {
+  if (!aAccessible) return LayoutDeviceIntPoint();
 
-  nsIFrame* parentFrame = parent->GetFrame();
-  if (!parentFrame) return nsIntPoint(0, 0);
+  if (Accessible* parent = aAccessible->Parent()) {
+    LayoutDeviceIntRect parentBounds = parent->Bounds();
+    // The rect returned from Bounds() is already in dev
+    // pixels, so we don't need to do any conversion here.
+    return parentBounds.TopLeft();
+  }
 
-  nsRect rect = parentFrame->GetScreenRectInAppUnits();
-  return nsPoint(rect.X(), rect.Y())
-      .ToNearestPixels(parentFrame->PresContext()->AppUnitsPerDevPixel());
+  return LayoutDeviceIntPoint();
+}
+
+LayoutDeviceIntPoint nsAccUtils::GetScreenCoordsForWindow(
+    Accessible* aAccessible) {
+  a11y::LocalAccessible* localAcc = aAccessible->AsLocal();
+  if (!localAcc) {
+    localAcc = aAccessible->AsRemote()->OuterDocOfRemoteBrowser();
+  }
+
+  LayoutDeviceIntPoint coords(0, 0);
+  nsCOMPtr<nsIDocShellTreeItem> treeItem(
+      nsCoreUtils::GetDocShellFor(localAcc->GetNode()));
+  if (!treeItem) return coords;
+
+  nsCOMPtr<nsIDocShellTreeOwner> treeOwner;
+  treeItem->GetTreeOwner(getter_AddRefs(treeOwner));
+  if (!treeOwner) return coords;
+
+  nsCOMPtr<nsIBaseWindow> baseWindow = do_QueryInterface(treeOwner);
+  if (baseWindow) {
+    baseWindow->GetPosition(&coords.x, &coords.y);  // in device pixels
+  }
+
+  return coords;
 }
 
 bool nsAccUtils::GetLiveAttrValue(uint32_t aRule, nsAString& aValue) {
   switch (aRule) {
     case eOffLiveAttr:
-      aValue = NS_LITERAL_STRING("off");
+      aValue = u"off"_ns;
       return true;
     case ePoliteLiveAttr:
-      aValue = NS_LITERAL_STRING("polite");
+      aValue = u"polite"_ns;
+      return true;
+    case eAssertiveLiveAttr:
+      aValue = u"assertive"_ns;
       return true;
   }
 
@@ -369,7 +363,7 @@ bool nsAccUtils::GetLiveAttrValue(uint32_t aRule, nsAString& aValue) {
 
 #ifdef DEBUG
 
-bool nsAccUtils::IsTextInterfaceSupportCorrect(Accessible* aAccessible) {
+bool nsAccUtils::IsTextInterfaceSupportCorrect(LocalAccessible* aAccessible) {
   // Don't test for accessible docs, it makes us create accessibles too
   // early and fire mutation events before we need to
   if (aAccessible->IsDoc()) return true;
@@ -377,8 +371,8 @@ bool nsAccUtils::IsTextInterfaceSupportCorrect(Accessible* aAccessible) {
   bool foundText = false;
   uint32_t childCount = aAccessible->ChildCount();
   for (uint32_t childIdx = 0; childIdx < childCount; childIdx++) {
-    Accessible* child = aAccessible->GetChildAt(childIdx);
-    if (child->IsText()) {
+    LocalAccessible* child = aAccessible->LocalChildAt(childIdx);
+    if (child && child->IsText()) {
       foundText = true;
       break;
     }
@@ -393,8 +387,19 @@ uint32_t nsAccUtils::TextLength(Accessible* aAccessible) {
     return 1;
   }
 
-  TextLeafAccessible* textLeaf = aAccessible->AsTextLeaf();
-  if (textLeaf) return textLeaf->Text().Length();
+  if (LocalAccessible* localAcc = aAccessible->AsLocal()) {
+    TextLeafAccessible* textLeaf = localAcc->AsTextLeaf();
+    if (textLeaf) {
+      return textLeaf->Text().Length();
+    }
+  } else if (aAccessible->IsText()) {
+    MOZ_ASSERT(StaticPrefs::accessibility_cache_enabled_AtStartup(),
+               "Shouldn't be called on a RemoteAccessible unless the cache is "
+               "enabled");
+    RemoteAccessible* remoteAcc = aAccessible->AsRemote();
+    MOZ_ASSERT(remoteAcc);
+    return remoteAcc->GetCachedTextLength();
+  }
 
   // For list bullets (or anything other accessible which would compute its own
   // text. They don't have their own frame.
@@ -405,9 +410,9 @@ uint32_t nsAccUtils::TextLength(Accessible* aAccessible) {
   return text.Length();
 }
 
-bool nsAccUtils::MustPrune(AccessibleOrProxy aAccessible) {
-  MOZ_ASSERT(!aAccessible.IsNull());
-  roles::Role role = aAccessible.Role();
+bool nsAccUtils::MustPrune(Accessible* aAccessible) {
+  MOZ_ASSERT(aAccessible);
+  roles::Role role = aAccessible->Role();
 
   if (role == roles::SLIDER) {
     // Always prune the tree for sliders, as it doesn't make sense for a
@@ -425,49 +430,17 @@ bool nsAccUtils::MustPrune(AccessibleOrProxy aAccessible) {
     return false;
   }
 
-  if (aAccessible.ChildCount() != 1) {
+  if (aAccessible->ChildCount() != 1) {
     // If the accessible has more than one child, don't prune it.
     return false;
   }
 
-  roles::Role childRole = aAccessible.FirstChild().Role();
+  roles::Role childRole = aAccessible->FirstChild()->Role();
   // If the accessible's child is a text leaf, prune the accessible.
   return childRole == roles::TEXT_LEAF || childRole == roles::STATICTEXT;
 }
 
-bool nsAccUtils::PersistentPropertiesToArray(nsIPersistentProperties* aProps,
-                                             nsTArray<Attribute>* aAttributes) {
-  if (!aProps) {
-    return true;
-  }
-  nsCOMPtr<nsISimpleEnumerator> propEnum;
-  nsresult rv = aProps->Enumerate(getter_AddRefs(propEnum));
-  NS_ENSURE_SUCCESS(rv, false);
-
-  bool hasMore;
-  while (NS_SUCCEEDED(propEnum->HasMoreElements(&hasMore)) && hasMore) {
-    nsCOMPtr<nsISupports> sup;
-    rv = propEnum->GetNext(getter_AddRefs(sup));
-    NS_ENSURE_SUCCESS(rv, false);
-
-    nsCOMPtr<nsIPropertyElement> propElem(do_QueryInterface(sup));
-    NS_ENSURE_TRUE(propElem, false);
-
-    nsAutoCString name;
-    rv = propElem->GetKey(name);
-    NS_ENSURE_SUCCESS(rv, false);
-
-    nsAutoString value;
-    rv = propElem->GetValue(value);
-    NS_ENSURE_SUCCESS(rv, false);
-
-    aAttributes->AppendElement(Attribute(name, value));
-  }
-
-  return true;
-}
-
-bool nsAccUtils::IsARIALive(const Accessible* aAccessible) {
+bool nsAccUtils::IsARIALive(const LocalAccessible* aAccessible) {
   // Get computed aria-live property based on the closest container with the
   // attribute. Inner nodes override outer nodes within the same
   // document.
@@ -494,7 +467,7 @@ bool nsAccUtils::IsARIALive(const Accessible* aAccessible) {
     } else if (role) {
       GetLiveAttrValue(role->liveAttRule, live);
     } else if (nsStaticAtom* value = GetAccService()->MarkupAttribute(
-                   ancestor, nsGkAtoms::live)) {
+                   ancestor, nsGkAtoms::aria_live)) {
       value->ToString(live);
     }
     if (!live.IsEmpty() && !live.EqualsLiteral("off")) {
@@ -513,3 +486,28 @@ bool nsAccUtils::IsARIALive(const Accessible* aAccessible) {
 
   return false;
 }
+
+Accessible* nsAccUtils::DocumentFor(Accessible* aAcc) {
+  if (!aAcc) {
+    return nullptr;
+  }
+  if (LocalAccessible* localAcc = aAcc->AsLocal()) {
+    return localAcc->Document();
+  }
+  return aAcc->AsRemote()->Document();
+}
+
+Accessible* nsAccUtils::GetAccessibleByID(Accessible* aDoc, uint64_t aID) {
+  if (!aDoc) {
+    return nullptr;
+  }
+  if (LocalAccessible* localAcc = aDoc->AsLocal()) {
+    if (DocAccessible* doc = localAcc->AsDoc()) {
+      return doc->GetAccessibleByUniqueID(
+          reinterpret_cast<void*>(static_cast<uintptr_t>(aID)));
+    }
+  } else if (DocAccessibleParent* doc = aDoc->AsRemote()->AsDoc()) {
+    return doc->GetAccessible(aID);
+  }
+  return nullptr;
+}
