@@ -6,7 +6,8 @@
 #ifndef nsAccUtils_h_
 #define nsAccUtils_h_
 
-#include "mozilla/a11y/Accessible.h"
+#include "mozilla/a11y/LocalAccessible.h"
+#include "mozilla/a11y/DocManager.h"
 
 #include "nsAccessibilityService.h"
 #include "nsCoreUtils.h"
@@ -16,6 +17,8 @@
 
 namespace mozilla {
 
+class PresShell;
+
 namespace dom {
 class Element;
 }
@@ -24,49 +27,15 @@ namespace a11y {
 
 class HyperTextAccessible;
 class DocAccessible;
+class Attribute;
 
 class nsAccUtils {
  public:
-  /**
-   * Returns value of attribute from the given attributes container.
-   *
-   * @param aAttributes - attributes container
-   * @param aAttrName - the name of requested attribute
-   * @param aAttrValue - value of attribute
-   */
-  static void GetAccAttr(nsIPersistentProperties* aAttributes,
-                         nsAtom* aAttrName, nsAString& aAttrValue);
-
-  /**
-   * Set value of attribute for the given attributes container.
-   *
-   * @param aAttributes - attributes container
-   * @param aAttrName - the name of requested attribute
-   * @param aAttrValue - new value of attribute
-   */
-  static void SetAccAttr(nsIPersistentProperties* aAttributes,
-                         nsAtom* aAttrName, const nsAString& aAttrValue);
-
-  static void SetAccAttr(nsIPersistentProperties* aAttributes,
-                         nsAtom* aAttrName, nsAtom* aAttrValue);
-
   /**
    * Set group attributes ('level', 'setsize', 'posinset').
    */
-  static void SetAccGroupAttrs(nsIPersistentProperties* aAttributes,
-                               int32_t aLevel, int32_t aSetSize,
-                               int32_t aPosInSet);
-
-  /**
-   * Get default value of the level for the given accessible.
-   */
-  static int32_t GetDefaultLevel(Accessible* aAcc);
-
-  /**
-   * Return ARIA level value or the default one if ARIA is missed for the
-   * given accessible.
-   */
-  static int32_t GetARIAOrDefaultLevel(Accessible* aAccessible);
+  static void SetAccGroupAttrs(AccAttributes* aAttributes, int32_t aLevel,
+                               int32_t aSetSize, int32_t aPosInSet);
 
   /**
    * Compute group level for nsIDOMXULContainerItemElement node.
@@ -78,11 +47,9 @@ class nsAccUtils {
    *
    * @param aAttributes    where to store the attributes
    * @param aStartContent  node to start from
-   * @param aTopContent    node to end at
    */
-  static void SetLiveContainerAttributes(nsIPersistentProperties* aAttributes,
-                                         nsIContent* aStartContent,
-                                         mozilla::dom::Element* aTopEl);
+  static void SetLiveContainerAttributes(AccAttributes* aAttributes,
+                                         nsIContent* aStartContent);
 
   /**
    * Any ARIA property of type boolean or NMTOKEN is undefined if the ARIA
@@ -96,14 +63,23 @@ class nsAccUtils {
   /**
    * Return atomic value of ARIA attribute of boolean or NMTOKEN type.
    */
-  static nsAtom* GetARIAToken(mozilla::dom::Element* aElement, nsAtom* aAttr);
+  static nsStaticAtom* GetARIAToken(mozilla::dom::Element* aElement,
+                                    nsAtom* aAttr);
+
+  /**
+   * If the given ARIA attribute has a specific known token value, return it.
+   * If the specification demands for a fallback value for unknown attribute
+   * values, return that. For all others, return a nullptr.
+   */
+  static nsStaticAtom* NormalizeARIAToken(mozilla::dom::Element* aElement,
+                                          nsAtom* aAttr);
 
   /**
    * Return document accessible for the given DOM node.
    */
   static DocAccessible* GetDocAccessibleFor(nsINode* aNode) {
-    nsIPresShell* presShell = nsCoreUtils::GetPresShellFor(aNode);
-    return GetAccService()->GetDocAccessible(presShell);
+    return GetAccService()->GetDocAccessible(
+        nsCoreUtils::GetPresShellFor(aNode));
   }
 
   /**
@@ -120,8 +96,8 @@ class nsAccUtils {
    * @param  aAccessible  [in] the item accessible
    * @param  aState       [in] the state of the item accessible
    */
-  static Accessible* GetSelectableContainer(Accessible* aAccessible,
-                                            uint64_t aState);
+  static LocalAccessible* GetSelectableContainer(LocalAccessible* aAccessible,
+                                                 uint64_t aState);
 
   /**
    * Return a text container accessible for the given node.
@@ -129,34 +105,51 @@ class nsAccUtils {
   static HyperTextAccessible* GetTextContainer(nsINode* aNode);
 
   static Accessible* TableFor(Accessible* aRow);
+  static LocalAccessible* TableFor(LocalAccessible* aRow);
+
+  /**
+   * Return true if the DOM node of a given accessible has a given attribute
+   * with a value of "true".
+   */
+  static bool IsDOMAttrTrue(const LocalAccessible* aAccessible, nsAtom* aAttr);
 
   /**
    * Return true if the DOM node of given accessible has aria-selected="true"
    * attribute.
    */
-  static bool IsARIASelected(Accessible* aAccessible);
+  static inline bool IsARIASelected(const LocalAccessible* aAccessible) {
+    return IsDOMAttrTrue(aAccessible, nsGkAtoms::aria_selected);
+  }
+
+  /**
+   * Return true if the DOM node of given accessible has
+   * aria-multiselectable="true" attribute.
+   */
+  static inline bool IsARIAMultiSelectable(const LocalAccessible* aAccessible) {
+    return IsDOMAttrTrue(aAccessible, nsGkAtoms::aria_multiselectable);
+  }
 
   /**
    * Converts the given coordinates to coordinates relative screen.
    *
-   * @param aX               [in] the given x coord
-   * @param aY               [in] the given y coord
+   * @param aX               [in] the given x coord in dev pixels
+   * @param aY               [in] the given y coord in dev pixels
    * @param aCoordinateType  [in] specifies coordinates origin (refer to
    *                         nsIAccessibleCoordinateType)
    * @param aAccessible      [in] the accessible if coordinates are given
    *                         relative it.
    * @return converted coordinates
    */
-  static nsIntPoint ConvertToScreenCoords(int32_t aX, int32_t aY,
-                                          uint32_t aCoordinateType,
-                                          Accessible* aAccessible);
+  static LayoutDeviceIntPoint ConvertToScreenCoords(int32_t aX, int32_t aY,
+                                                    uint32_t aCoordinateType,
+                                                    Accessible* aAccessible);
 
   /**
    * Converts the given coordinates relative screen to another coordinate
    * system.
    *
-   * @param aX               [in, out] the given x coord
-   * @param aY               [in, out] the given y coord
+   * @param aX               [in, out] the given x coord in dev pixels
+   * @param aY               [in, out] the given y coord in dev pixels
    * @param aCoordinateType  [in] specifies coordinates origin (refer to
    *                         nsIAccessibleCoordinateType)
    * @param aAccessible      [in] the accessible if coordinates are given
@@ -167,11 +160,21 @@ class nsAccUtils {
                                     Accessible* aAccessible);
 
   /**
-   * Returns coordinates relative screen for the parent of the given accessible.
+   * Returns screen-relative coordinates (in dev pixels) for the parent of the
+   * given accessible.
    *
    * @param [in] aAccessible  the accessible
    */
-  static nsIntPoint GetScreenCoordsForParent(Accessible* aAccessible);
+  static LayoutDeviceIntPoint GetScreenCoordsForParent(Accessible* aAccessible);
+
+  /**
+   * Returns coordinates in device pixels relative screen for the top level
+   * window.
+   *
+   * @param aAccessible the acc hosted in the window.
+   */
+  static mozilla::LayoutDeviceIntPoint GetScreenCoordsForWindow(
+      mozilla::a11y::Accessible* aAccessible);
 
   /**
    * Get the 'live' or 'container-live' object attribute value from the given
@@ -189,7 +192,7 @@ class nsAccUtils {
    * Detect whether the given accessible object implements nsIAccessibleText,
    * when it is text or has text child node.
    */
-  static bool IsTextInterfaceSupportCorrect(Accessible* aAccessible);
+  static bool IsTextInterfaceSupportCorrect(LocalAccessible* aAccessible);
 #endif
 
   /**
@@ -217,7 +220,7 @@ class nsAccUtils {
   static uint32_t To32States(uint64_t aState, bool* aIsExtra) {
     uint32_t extraState = aState >> 31;
     *aIsExtra = !!extraState;
-    return aState | extraState;
+    return extraState ? extraState : aState;
   }
 
   /**
@@ -225,6 +228,29 @@ class nsAccUtils {
    * to platform accessibility APIs, should the children be pruned off?
    */
   static bool MustPrune(Accessible* aAccessible);
+
+  /**
+   * Return true if the given accessible is within an ARIA live region; i.e.
+   * the container-live attribute would be something other than "off" or empty.
+   */
+  static bool IsARIALive(const LocalAccessible* aAccessible);
+
+  /**
+   * Get the document Accessible which owns a given Accessible.
+   * This function is needed because there is no unified base class for local
+   * and remote documents.
+   * If aAcc is null, null will be returned.
+   */
+  static Accessible* DocumentFor(Accessible* aAcc);
+
+  /**
+   * Get an Accessible in a given document by its unique id.
+   * An Accessible's id can be obtained using Accessible::ID.
+   * This function is needed because there is no unified base class for local
+   * and remote documents.
+   * If aDoc is nul, null will be returned.
+   */
+  static Accessible* GetAccessibleByID(Accessible* aDoc, uint64_t aID);
 };
 
 }  // namespace a11y
