@@ -16,6 +16,7 @@
 
 #include "nsCOMPtr.h"
 #include "mozilla/dom/HTMLOptionElement.h"
+#include "mozilla/dom/HTMLOptGroupElement.h"
 #include "mozilla/dom/HTMLSelectElement.h"
 #include "nsComboboxControlFrame.h"
 #include "nsContainerFrame.h"
@@ -98,6 +99,11 @@ bool HTMLSelectListAccessible::IsAcceptableChild(nsIContent* aEl) const {
   return aEl->IsAnyOfHTMLElements(nsGkAtoms::option, nsGkAtoms::optgroup);
 }
 
+bool HTMLSelectListAccessible::AttributeChangesState(nsAtom* aAttribute) {
+  return aAttribute == nsGkAtoms::multiple ||
+         LocalAccessible::AttributeChangesState(aAttribute);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // HTMLSelectOptionAccessible
 ////////////////////////////////////////////////////////////////////////////////
@@ -116,21 +122,35 @@ role HTMLSelectOptionAccessible::NativeRole() const {
 }
 
 ENameValueFlag HTMLSelectOptionAccessible::NativeName(nsString& aName) const {
-  // CASE #1 -- great majority of the cases
-  // find the label attribute - this is what the W3C says we should use
-  mContent->AsElement()->GetAttr(kNameSpaceID_None, nsGkAtoms::label, aName);
-  if (!aName.IsEmpty()) return eNameOK;
-
-  // CASE #2 -- no label parameter, get the first child,
-  // use it if it is a text node
-  nsIContent* text = mContent->GetFirstChild();
-  if (text && text->IsText()) {
-    nsTextEquivUtils::AppendTextEquivFromTextContent(text, &aName);
-    aName.CompressWhitespace();
+  if (auto* option = dom::HTMLOptionElement::FromNode(mContent)) {
+    option->GetAttr(nsGkAtoms::label, aName);
+    if (!aName.IsEmpty()) {
+      return eNameOK;
+    }
+    option->GetText(aName);
+    return eNameFromSubtree;
+  }
+  if (auto* group = dom::HTMLOptGroupElement::FromNode(mContent)) {
+    group->GetLabel(aName);
     return aName.IsEmpty() ? eNameOK : eNameFromSubtree;
   }
+  MOZ_ASSERT_UNREACHABLE("What content do we have?");
+  return eNameFromSubtree;
+}
 
-  return eNameOK;
+void HTMLSelectOptionAccessible::DOMAttributeChanged(
+    int32_t aNameSpaceID, nsAtom* aAttribute, int32_t aModType,
+    const nsAttrValue* aOldValue, uint64_t aOldState) {
+  HyperTextAccessibleWrap::DOMAttributeChanged(aNameSpaceID, aAttribute,
+                                               aModType, aOldValue, aOldState);
+
+  if (aAttribute == nsGkAtoms::label) {
+    dom::Element* elm = Elm();
+    if (!elm->HasAttr(kNameSpaceID_None, nsGkAtoms::aria_labelledby) &&
+        !elm->HasAttr(kNameSpaceID_None, nsGkAtoms::aria_label)) {
+      mDoc->FireDelayedEvent(nsIAccessibleEvent::EVENT_NAME_CHANGE, this);
+    }
+  }
 }
 
 uint64_t HTMLSelectOptionAccessible::NativeState() const {
@@ -174,8 +194,8 @@ uint64_t HTMLSelectOptionAccessible::NativeState() const {
     // <select> is not collapsed: compare bounds to calculate OFFSCREEN
     LocalAccessible* listAcc = LocalParent();
     if (listAcc) {
-      nsIntRect optionRect = Bounds();
-      nsIntRect listRect = listAcc->Bounds();
+      LayoutDeviceIntRect optionRect = Bounds();
+      LayoutDeviceIntRect listRect = listAcc->Bounds();
       if (optionRect.Y() < listRect.Y() ||
           optionRect.YMost() > listRect.YMost()) {
         state |= states::OFFSCREEN;
@@ -191,19 +211,6 @@ uint64_t HTMLSelectOptionAccessible::NativeInteractiveState() const {
                                : states::FOCUSABLE | states::SELECTABLE;
 }
 
-int32_t HTMLSelectOptionAccessible::GetLevelInternal() {
-  nsIContent* parentContent = mContent->GetParent();
-
-  int32_t level =
-      parentContent->NodeInfo()->Equals(nsGkAtoms::optgroup) ? 2 : 1;
-
-  if (level == 1 && Role() != roles::HEADING) {
-    level = 0;  // In a single level list, the level is irrelevant
-  }
-
-  return level;
-}
-
 nsRect HTMLSelectOptionAccessible::RelativeBounds(
     nsIFrame** aBoundingFrame) const {
   LocalAccessible* combobox = GetCombobox();
@@ -214,19 +221,29 @@ nsRect HTMLSelectOptionAccessible::RelativeBounds(
   return HyperTextAccessibleWrap::RelativeBounds(aBoundingFrame);
 }
 
+nsresult HTMLSelectOptionAccessible::HandleAccEvent(AccEvent* aEvent) {
+  nsresult rv = HyperTextAccessibleWrap::HandleAccEvent(aEvent);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  AccStateChangeEvent* event = downcast_accEvent(aEvent);
+  if (event && (event->GetState() == states::SELECTED)) {
+    LocalAccessible* widget = ContainerWidget();
+    if (widget && !widget->AreItemsOperable()) {
+      // Collapsed options' ACTIVE state reflects their SELECT state.
+      nsEventShell::FireEvent(this, states::ACTIVE, event->IsStateEnabled(),
+                              true);
+    }
+  }
+
+  return NS_OK;
+}
+
 void HTMLSelectOptionAccessible::ActionNameAt(uint8_t aIndex,
                                               nsAString& aName) {
   if (aIndex == eAction_Select) aName.AssignLiteral("select");
 }
 
-uint8_t HTMLSelectOptionAccessible::ActionCount() const { return 1; }
-
-bool HTMLSelectOptionAccessible::DoAction(uint8_t aIndex) const {
-  if (aIndex != eAction_Select) return false;
-
-  DoCommand();
-  return true;
-}
+bool HTMLSelectOptionAccessible::HasPrimaryAction() const { return true; }
 
 void HTMLSelectOptionAccessible::SetSelected(bool aSelect) {
   HTMLOptionElement* option = HTMLOptionElement::FromNode(mContent);
@@ -261,16 +278,7 @@ bool HTMLSelectOptGroupAccessible::IsAcceptableChild(nsIContent* aEl) const {
   return aEl->IsCharacterData() || aEl->IsHTMLElement(nsGkAtoms::option);
 }
 
-uint8_t HTMLSelectOptGroupAccessible::ActionCount() const { return 0; }
-
-void HTMLSelectOptGroupAccessible::ActionNameAt(uint8_t aIndex,
-                                                nsAString& aName) {
-  aName.Truncate();
-}
-
-bool HTMLSelectOptGroupAccessible::DoAction(uint8_t aIndex) const {
-  return false;
-}
+bool HTMLSelectOptGroupAccessible::HasPrimaryAction() const { return false; }
 
 ////////////////////////////////////////////////////////////////////////////////
 // HTMLComboboxAccessible
@@ -283,14 +291,10 @@ HTMLComboboxAccessible::HTMLComboboxAccessible(nsIContent* aContent,
   mGenericTypes |= eCombobox;
   mStateFlags |= eNoKidsFromDOM;
 
-  nsComboboxControlFrame* comboFrame = do_QueryFrame(GetFrame());
-  if (comboFrame) {
-    nsIFrame* listFrame = comboFrame->GetDropDown();
-    if (listFrame) {
-      mListAccessible = new HTMLComboboxListAccessible(mParent, mContent, mDoc);
-      Document()->BindToDocument(mListAccessible, nullptr);
-      AppendChild(mListAccessible);
-    }
+  if ((nsComboboxControlFrame*)do_QueryFrame(GetFrame())) {
+    mListAccessible = new HTMLComboboxListAccessible(mParent, mContent, mDoc);
+    Document()->BindToDocument(mListAccessible, nullptr);
+    AppendChild(mListAccessible);
   }
 }
 
@@ -335,7 +339,7 @@ uint64_t HTMLComboboxAccessible::NativeState() const {
   return state;
 }
 
-void HTMLComboboxAccessible::Description(nsString& aDescription) {
+void HTMLComboboxAccessible::Description(nsString& aDescription) const {
   aDescription.Truncate();
   // First check to see if combo box itself has a description, perhaps through
   // tooltip (title attribute) or via aria-describedby
@@ -353,14 +357,7 @@ void HTMLComboboxAccessible::Value(nsString& aValue) const {
   if (option) option->Name(aValue);
 }
 
-uint8_t HTMLComboboxAccessible::ActionCount() const { return 1; }
-
-bool HTMLComboboxAccessible::DoAction(uint8_t aIndex) const {
-  if (aIndex != eAction_Click) return false;
-
-  DoCommand();
-  return true;
-}
+bool HTMLComboboxAccessible::HasPrimaryAction() const { return true; }
 
 void HTMLComboboxAccessible::ActionNameAt(uint8_t aIndex, nsAString& aName) {
   if (aIndex != HTMLComboboxAccessible::eAction_Click) return;
@@ -433,16 +430,6 @@ HTMLComboboxListAccessible::HTMLComboboxListAccessible(LocalAccessible* aParent,
 ////////////////////////////////////////////////////////////////////////////////
 // HTMLComboboxAccessible: LocalAccessible
 
-nsIFrame* HTMLComboboxListAccessible::GetFrame() const {
-  nsIFrame* frame = HTMLSelectListAccessible::GetFrame();
-  nsComboboxControlFrame* comboBox = do_QueryFrame(frame);
-  if (comboBox) {
-    return comboBox->GetDropDown();
-  }
-
-  return nullptr;
-}
-
 role HTMLComboboxListAccessible::NativeRole() const {
   return roles::COMBOBOX_LIST;
 }
