@@ -5,16 +5,17 @@
 "use strict";
 
 /* import-globals-from ../mochitest/common.js */
+/* import-globals-from ../mochitest/layout.js */
 /* import-globals-from ../mochitest/promisified-events.js */
 
-/* exported Logger, MOCHITESTS_DIR, invokeSetAttribute, invokeFocus,
+/* exported Logger, MOCHITESTS_DIR, isCacheEnabled, isWinNoCache, invokeSetAttribute, invokeFocus,
             invokeSetStyle, getAccessibleDOMNodeID, getAccessibleTagName,
             addAccessibleTask, findAccessibleChildByID, isDefunct,
             CURRENT_CONTENT_DIR, loadScripts, loadContentScripts, snippetToURL,
             Cc, Cu, arrayFromChildren, forceGC, contentSpawnMutation,
             DEFAULT_IFRAME_ID, DEFAULT_IFRAME_DOC_BODY_ID, invokeContentTask,
             matchContentDoc, currentContentDoc, getContentDPR,
-            waitForImageMap, getContentBoundsForDOMElm */
+            waitForImageMap, getContentBoundsForDOMElm, untilCacheIs, untilCacheOk, testBoundsInContent, waitForContentPaint */
 
 const CURRENT_FILE_DIR = "/browser/accessible/tests/browser/";
 
@@ -42,6 +43,15 @@ const DEFAULT_IFRAME_DOC_BODY_ID = "default-iframe-body-id";
 const HTML_MIME_TYPE = "text/html";
 const XHTML_MIME_TYPE = "application/xhtml+xml";
 
+const isCacheEnabled = Services.prefs.getBoolPref(
+  "accessibility.cache.enabled",
+  false
+);
+
+// Some RemoteAccessible methods aren't supported on Windows when the cache is
+// disabled.
+const isWinNoCache = !isCacheEnabled && AppConstants.platform == "win";
+
 function loadHTMLFromFile(path) {
   // Load the HTML to return in the response from file.
   // Since it's relative to the cwd of the test runner, we start there and
@@ -313,16 +323,8 @@ function loadScripts(...scripts) {
  *        a list of scripts to load into content
  */
 async function loadContentScripts(target, ...scripts) {
-  for (let script of scripts) {
-    let contentScript;
-    if (typeof script === "string") {
-      // If script string includes a .jsm extention, assume it is a module path.
-      contentScript = `${CURRENT_DIR}${script}`;
-    } else {
-      // Script is a object that has { dir, name } format.
-      contentScript = `${script.dir}${script.name}`;
-    }
-
+  for (let { script, symbol } of scripts) {
+    let contentScript = `${CURRENT_DIR}${script}`;
     let loadedScriptSet = LOADED_CONTENT_SCRIPTS.get(contentScript);
     if (!loadedScriptSet) {
       loadedScriptSet = new WeakSet();
@@ -331,9 +333,14 @@ async function loadContentScripts(target, ...scripts) {
       continue;
     }
 
-    await SpecialPowers.spawn(target, [contentScript], async _contentScript => {
-      ChromeUtils.import(_contentScript, content.window);
-    });
+    await SpecialPowers.spawn(
+      target,
+      [contentScript, symbol],
+      async (_contentScript, importSymbol) => {
+        let module = ChromeUtils.import(_contentScript);
+        content.window[importSymbol] = module[importSymbol];
+      }
+    );
     loadedScriptSet.add(target);
   }
 }
@@ -358,7 +365,8 @@ function wrapWithIFrame(doc, options = {}) {
     } else {
       srcURL.searchParams.append(
         "html",
-        `<html>
+        `<!doctype html>
+        <html>
           <head>
             <meta charset="utf-8"/>
             <title>Accessibility Fission Test</title>
@@ -377,7 +385,8 @@ function wrapWithIFrame(doc, options = {}) {
         `<body ${attrsToString(iframeDocBodyAttrs)}>`
       );
     } else {
-      doc = `<body ${attrsToString(iframeDocBodyAttrs)}>${doc}</body>`;
+      doc = `<!doctype html>
+      <body ${attrsToString(iframeDocBodyAttrs)}>${doc}</body>`;
     }
 
     src = `data:${mimeType};charset=utf-8,${encodeURIComponent(doc)}`;
@@ -414,7 +423,8 @@ function snippetToURL(doc, options = {}) {
   }
 
   const encodedDoc = encodeURIComponent(
-    `<html>
+    `<!doctype html>
+    <html>
       <head>
         <meta charset="utf-8"/>
         <title>Accessibility Test</title>
@@ -431,7 +441,11 @@ function accessibleTask(doc, task, options = {}) {
     gIsRemoteIframe = options.remoteIframe;
     gIsIframe = options.iframe || gIsRemoteIframe;
     let url;
-    if (doc.endsWith("html") && !gIsIframe) {
+    if (options.chrome && doc.endsWith("html")) {
+      // Load with a chrome:// URL so this loads as a chrome document in the
+      // parent process.
+      url = `${CURRENT_DIR}${doc}`;
+    } else if (doc.endsWith("html") && !gIsIframe) {
       url = `${CURRENT_CONTENT_DIR}${doc}`;
     } else {
       url = snippetToURL(doc, options);
@@ -445,10 +459,13 @@ function accessibleTask(doc, task, options = {}) {
       }
     });
 
-    const onContentDocLoad = waitForEvent(
-      EVENT_DOCUMENT_LOAD_COMPLETE,
-      DEFAULT_CONTENT_DOC_BODY_ID
-    );
+    let onContentDocLoad;
+    if (!options.chrome) {
+      onContentDocLoad = waitForEvent(
+        EVENT_DOCUMENT_LOAD_COMPLETE,
+        DEFAULT_CONTENT_DOC_BODY_ID
+      );
+    }
 
     let onIframeDocLoad;
     if (options.remoteIframe && !options.skipFissionDocLoad) {
@@ -461,7 +478,22 @@ function accessibleTask(doc, task, options = {}) {
     await BrowserTestUtils.withNewTab(
       {
         gBrowser,
-        url,
+        // For chrome, we need a non-remote browser.
+        opening: !options.chrome
+          ? url
+          : () => {
+              // Passing forceNotRemote: true still sets maychangeremoteness,
+              // which will cause data: URIs to load remotely. There's no way to
+              // avoid this with gBrowser or BrowserTestUtils. Therefore, we
+              // load a blank document initially and replace it below.
+              gBrowser.selectedTab = BrowserTestUtils.addTab(
+                gBrowser,
+                "about:blank",
+                {
+                  forceNotRemote: true,
+                }
+              );
+            },
       },
       async function(browser) {
         registerCleanupFunction(() => {
@@ -473,13 +505,45 @@ function accessibleTask(doc, task, options = {}) {
           }
         });
 
+        if (options.chrome) {
+          await SpecialPowers.pushPrefEnv({
+            set: [["security.allow_unsafe_parent_loads", true]],
+          });
+          // Ensure this never becomes a remote browser.
+          browser.removeAttribute("maychangeremoteness");
+          // Now we can load our page without it becoming remote.
+          browser.setAttribute("src", url);
+        }
+
         await SimpleTest.promiseFocus(browser);
-        await loadContentScripts(browser, "Common.jsm");
+        await loadContentScripts(browser, {
+          script: "Common.jsm",
+          symbol: "CommonUtils",
+        });
 
-        ok(Services.appinfo.browserTabsRemoteAutostart, "e10s enabled");
-        ok(browser.isRemoteBrowser, "Actually remote browser");
+        if (options.chrome) {
+          ok(!browser.isRemoteBrowser, "Not remote browser");
+        } else if (Services.appinfo.browserTabsRemoteAutostart) {
+          ok(browser.isRemoteBrowser, "Actually remote browser");
+        }
 
-        const { accessible: docAccessible } = await onContentDocLoad;
+        let docAccessible;
+        if (options.chrome) {
+          // Chrome documents don't fire DOCUMENT_LOAD_COMPLETE. Instead, wait
+          // until we can get the DocAccessible and it doesn't have the busy
+          // state.
+          await BrowserTestUtils.waitForCondition(() => {
+            docAccessible = getAccessible(browser.contentWindow.document);
+            if (!docAccessible) {
+              return false;
+            }
+            const state = {};
+            docAccessible.getState(state, {});
+            return !(state.value & STATE_BUSY);
+          });
+        } else {
+          ({ accessible: docAccessible } = await onContentDocLoad);
+        }
         let iframeDocAccessible;
         if (gIsIframe) {
           if (!options.skipFissionDocLoad) {
@@ -514,6 +578,13 @@ function accessibleTask(doc, task, options = {}) {
  *         - {Boolean} topLevel
  *           Flag to run the test with content in the top level content process.
  *           Default is true.
+ *         - {Boolean} chrome
+ *           Flag to run the test with content as a chrome document in the
+ *           parent process. Default is false. Although url can be a markup
+ *           snippet, a snippet cannot be used for XUL content. To load XUL,
+ *           specify a relative URL to a XUL document. In that case, toplevel
+ *           should usually be set to false, since XUL documents don't work in
+ *           content processes.
  *         - {Boolean} iframe
  *           Flag to run the test with content wrapped in an iframe. Default is
  *           false.
@@ -532,11 +603,28 @@ function accessibleTask(doc, task, options = {}) {
  *           a set of attributes to be applied to a iframe content document body
  */
 function addAccessibleTask(doc, task, options = {}) {
-  const { topLevel = true, iframe = false, remoteIframe = false } = options;
+  const {
+    topLevel = true,
+    chrome = false,
+    iframe = false,
+    remoteIframe = false,
+  } = options;
   if (topLevel) {
     add_task(
       accessibleTask(doc, task, {
         ...options,
+        chrome: false,
+        iframe: false,
+        remoteIframe: false,
+      })
+    );
+  }
+
+  if (chrome) {
+    add_task(
+      accessibleTask(doc, task, {
+        ...options,
+        topLevel: false,
         iframe: false,
         remoteIframe: false,
       })
@@ -548,6 +636,7 @@ function addAccessibleTask(doc, task, options = {}) {
       accessibleTask(doc, task, {
         ...options,
         topLevel: false,
+        chrome: false,
         remoteIframe: false,
       })
     );
@@ -558,6 +647,7 @@ function addAccessibleTask(doc, task, options = {}) {
       accessibleTask(doc, task, {
         ...options,
         topLevel: false,
+        chrome: false,
         iframe: false,
       })
     );
@@ -726,3 +816,111 @@ async function getContentBoundsForDOMElm(browser, id) {
     return LayoutUtils.getBoundsForDOMElm(contentId, content.document);
   });
 }
+
+const CACHE_WAIT_TIMEOUT_MS = 5000;
+
+/**
+ * Wait for a predicate to be true after cache ticks.
+ * This function takes two callbacks, the condition is evaluated
+ * by calling the first callback with the arguments returned by the second.
+ * This allows us to asynchronously return the arguments as a result if the condition
+ * of the first callback is met, or if it times out. The returned arguments can then
+ * be used to record a pass or fail in the test.
+ */
+function untilCacheCondition(conditionFunc, argsFunc) {
+  return new Promise((resolve, reject) => {
+    let args = argsFunc();
+    if (conditionFunc(...args)) {
+      resolve(args);
+      return;
+    }
+
+    let cacheObserver = {
+      observe(subject) {
+        args = argsFunc();
+        if (conditionFunc(...args)) {
+          clearTimeout(this.timer);
+          Services.obs.removeObserver(this, "accessible-cache");
+          resolve(args);
+        }
+      },
+
+      timeout() {
+        Services.obs.removeObserver(this, "accessible-cache");
+        args = argsFunc();
+        resolve(args);
+      },
+    };
+
+    cacheObserver.timer = setTimeout(
+      cacheObserver.timeout.bind(cacheObserver),
+      CACHE_WAIT_TIMEOUT_MS
+    );
+    Services.obs.addObserver(cacheObserver, "accessible-cache");
+  });
+}
+
+function untilCacheOk(conditionFunc, message) {
+  return untilCacheCondition(
+    (v, _unusedMessage) => v,
+    () => [conditionFunc(), message]
+  ).then(([v, msg]) => ok(v, msg));
+}
+
+function untilCacheIs(retrievalFunc, expected, message) {
+  return untilCacheCondition(
+    (a, b, _unusedMessage) => Object.is(a, b),
+    () => [retrievalFunc(), expected, message]
+  ).then(([got, exp, msg]) => is(got, exp, msg));
+}
+
+async function waitForContentPaint(browser) {
+  await SpecialPowers.spawn(browser, [], () => {
+    return new Promise(function(r) {
+      content.requestAnimationFrame(() => content.setTimeout(r));
+    });
+  });
+}
+
+async function testBoundsInContent(iframeDocAcc, id, browser) {
+  const acc = findAccessibleChildByID(iframeDocAcc, id);
+  const x = {};
+  const y = {};
+  const width = {};
+  const height = {};
+  acc.getBounds(x, y, width, height);
+
+  await invokeContentTask(
+    browser,
+    [id, x.value, y.value, width.value, height.value],
+    (_id, _x, _y, _width, _height) => {
+      const { Layout: LayoutUtils } = ChromeUtils.import(
+        "chrome://mochitests/content/browser/accessible/tests/browser/Layout.jsm"
+      );
+
+      let [
+        expectedX,
+        expectedY,
+        expectedWidth,
+        expectedHeight,
+      ] = LayoutUtils.getBoundsForDOMElm(_id, content.document);
+
+      ok(
+        _x >= expectedX - 5 && _x <= expectedX + 5,
+        "Got " + _x + ", accurate x for " + _id
+      );
+      ok(
+        _y >= expectedY - 5 && _y <= expectedY + 5,
+        "Got " + _y + ", accurate y for " + _id
+      );
+      ok(
+        _width >= expectedWidth - 5 && _width <= expectedWidth + 5,
+        "Got " + _width + ", accurate width for " + _id
+      );
+      ok(
+        _height >= expectedHeight - 5 && _height <= expectedHeight + 5,
+        "Got " + _height + ", accurate height for " + _id
+      );
+    }
+  );
+}
