@@ -1,4 +1,5 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -29,8 +30,8 @@ class XULTreeGridAccessible : public XULTreeAccessible, public TableAccessible {
   // TableAccessible
   virtual uint32_t ColCount() const override;
   virtual uint32_t RowCount() override;
-  virtual Accessible* CellAt(uint32_t aRowIndex,
-                             uint32_t aColumnIndex) override;
+  virtual LocalAccessible* CellAt(uint32_t aRowIndex,
+                                  uint32_t aColumnIndex) override;
   virtual void ColDescription(uint32_t aColIdx,
                               nsString& aDescription) override;
   virtual bool IsColSelected(uint32_t aColIdx) override;
@@ -45,9 +46,9 @@ class XULTreeGridAccessible : public XULTreeAccessible, public TableAccessible {
   virtual void SelectedRowIndices(nsTArray<uint32_t>* aRows) override;
   virtual void SelectRow(uint32_t aRowIdx) override;
   virtual void UnselectRow(uint32_t aRowIdx) override;
-  virtual Accessible* AsAccessible() override { return this; }
+  virtual LocalAccessible* AsAccessible() override { return this; }
 
-  // Accessible
+  // LocalAccessible
   virtual TableAccessible* AsTable() override { return this; }
   virtual a11y::role NativeRole() const override;
 
@@ -55,7 +56,7 @@ class XULTreeGridAccessible : public XULTreeAccessible, public TableAccessible {
   virtual ~XULTreeGridAccessible();
 
   // XULTreeAccessible
-  virtual already_AddRefed<Accessible> CreateTreeItemAccessible(
+  virtual already_AddRefed<LocalAccessible> CreateTreeItemAccessible(
       int32_t aRow) const override;
 };
 
@@ -65,10 +66,10 @@ class XULTreeGridAccessible : public XULTreeAccessible, public TableAccessible {
  */
 class XULTreeGridRowAccessible final : public XULTreeItemAccessibleBase {
  public:
-  using Accessible::GetChildAt;
+  using LocalAccessible::LocalChildAt;
 
   XULTreeGridRowAccessible(nsIContent* aContent, DocAccessible* aDoc,
-                           Accessible* aParent, dom::XULTreeElement* aTree,
+                           LocalAccessible* aParent, dom::XULTreeElement* aTree,
                            nsITreeView* aTreeView, int32_t aRow);
 
   // nsISupports and cycle collection
@@ -76,14 +77,14 @@ class XULTreeGridRowAccessible final : public XULTreeItemAccessibleBase {
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(XULTreeGridRowAccessible,
                                            XULTreeItemAccessibleBase)
 
-  // Accessible
+  // LocalAccessible
   virtual void Shutdown() override;
   virtual a11y::role NativeRole() const override;
   virtual ENameValueFlag Name(nsString& aName) const override;
-  virtual Accessible* ChildAtPoint(int32_t aX, int32_t aY,
-                                   EWhichChildAtPoint aWhichChild) override;
+  virtual LocalAccessible* LocalChildAtPoint(
+      int32_t aX, int32_t aY, EWhichChildAtPoint aWhichChild) override;
 
-  virtual Accessible* GetChildAt(uint32_t aIndex) const override;
+  virtual LocalAccessible* LocalChildAt(uint32_t aIndex) const override;
   virtual uint32_t ChildCount() const override;
 
   // XULTreeItemAccessibleBase
@@ -118,14 +119,14 @@ class XULTreeGridCellAccessible : public LeafAccessible,
   NS_DECL_CYCLE_COLLECTION_CLASS_INHERITED(XULTreeGridCellAccessible,
                                            LeafAccessible)
 
-  // Accessible
+  // LocalAccessible
   virtual void Shutdown() override;
   virtual TableCellAccessible* AsTableCell() override { return this; }
   virtual nsRect BoundsInAppUnits() const override;
   virtual nsIntRect BoundsInCSSPixels() const override;
   virtual ENameValueFlag Name(nsString& aName) const override;
-  virtual Accessible* FocusedChild() override;
-  virtual already_AddRefed<nsIPersistentProperties> NativeAttributes() override;
+  virtual LocalAccessible* FocusedChild() override;
+  virtual already_AddRefed<AccAttributes> NativeAttributes() override;
   virtual int32_t IndexInParent() const override;
   virtual Relation RelationByType(RelationType aType) const override;
   virtual a11y::role NativeRole() const override;
@@ -133,9 +134,8 @@ class XULTreeGridCellAccessible : public LeafAccessible,
   virtual uint64_t NativeInteractiveState() const override;
 
   // ActionAccessible
-  virtual uint8_t ActionCount() const override;
+  virtual bool HasPrimaryAction() const override;
   virtual void ActionNameAt(uint8_t aIndex, nsAString& aName) override;
-  virtual bool DoAction(uint8_t aIndex) const override;
 
   // TableCellAccessible
   virtual TableAccessible* Table() const override;
@@ -155,8 +155,8 @@ class XULTreeGridCellAccessible : public LeafAccessible,
  protected:
   virtual ~XULTreeGridCellAccessible();
 
-  // Accessible
-  virtual Accessible* GetSiblingAtOffset(
+  // LocalAccessible
+  virtual LocalAccessible* GetSiblingAtOffset(
       int32_t aOffset, nsresult* aError = nullptr) const override;
   MOZ_CAN_RUN_SCRIPT
   virtual void DispatchClickEvent(nsIContent* aContent,
