@@ -13,8 +13,8 @@
 #include "nsCOMPtr.h"
 #include "nsCSSPseudoElements.h"
 
-class nsMeterFrame : public nsContainerFrame,
-                     public nsIAnonymousContentCreator
+class nsMeterFrame final : public nsContainerFrame,
+                           public nsIAnonymousContentCreator
 
 {
   typedef mozilla::dom::Element Element;
@@ -23,7 +23,7 @@ class nsMeterFrame : public nsContainerFrame,
   NS_DECL_QUERYFRAME
   NS_DECL_FRAMEARENA_HELPERS(nsMeterFrame)
 
-  explicit nsMeterFrame(nsStyleContext* aContext);
+  explicit nsMeterFrame(ComputedStyle* aStyle, nsPresContext* aPresContext);
   virtual ~nsMeterFrame();
 
   virtual void DestroyFrom(nsIFrame* aDestructRoot,
@@ -35,7 +35,7 @@ class nsMeterFrame : public nsContainerFrame,
 
 #ifdef DEBUG_FRAME_DUMP
   virtual nsresult GetFrameName(nsAString& aResult) const override {
-    return MakeFrameName(NS_LITERAL_STRING("Meter"), aResult);
+    return MakeFrameName(u"Meter"_ns, aResult);
   }
 #endif
 
@@ -51,8 +51,10 @@ class nsMeterFrame : public nsContainerFrame,
   virtual mozilla::LogicalSize ComputeAutoSize(
       gfxContext* aRenderingContext, mozilla::WritingMode aWM,
       const mozilla::LogicalSize& aCBSize, nscoord aAvailableISize,
-      const mozilla::LogicalSize& aMargin, const mozilla::LogicalSize& aBorder,
-      const mozilla::LogicalSize& aPadding, ComputeSizeFlags aFlags) override;
+      const mozilla::LogicalSize& aMargin,
+      const mozilla::LogicalSize& aBorderPadding,
+      const mozilla::StyleSizeOverrides& aSizeOverrides,
+      mozilla::ComputeSizeFlags aFlags) override;
 
   virtual nscoord GetMinISize(gfxContext* aRenderingContext) override;
   virtual nscoord GetPrefISize(gfxContext* aRenderingContext) override;
@@ -67,9 +69,6 @@ class nsMeterFrame : public nsContainerFrame,
    */
   bool ShouldUseNativeStyle() const;
 
-  virtual Element* GetPseudoElement(
-      mozilla::CSSPseudoElementType aType) override;
-
  protected:
   // Helper function which reflow the anonymous div frame.
   void ReflowBarFrame(nsIFrame* aBarFrame, nsPresContext* aPresContext,
